import importlib.util
import os
import sys

CONFIG_NAME = "config.py"

def find_config():
    """Search for config.py in the user's project."""
    for path in sys.path:
        config_path = os.path.join(path, CONFIG_NAME)

        if os.path.exists(config_path):
            return config_path
    return None

def load_config():
    """Dynamically import config.py if it exists."""
    config_path = find_config()
    if not config_path:
        raise FileNotFoundError("No config.py found. Please create one in your project root.")

    # create a module specification for config.py with naming config
    # importlib.util.spec_from_file_location(module name, import path)
    # type = ModuleSpec
    spec = importlib.util.spec_from_file_location("config", config_path)

    # create an empty module class instance from specification.
    # importlib.util.module_from_spec(spec)
    config = importlib.util.module_from_spec(spec)

    # execute an empty module
    # load and execute config.py from config
    spec.loader.exec_module(config)

    # Inject into `sys.modules` as `mizuhara.config`
    sys.modules["mizuhara.config"] = config
    return config


import importlib.resources
from yaml import safe_load
from os.path import exists


def translate(domain: str, key: str, language_code: str = "en") -> str:
    """
    this function is charge of translate string, which is defined on yaml file in the same path, into another language_code.
    if you need customize, please create another yaml file on translation folder,
    and use this function to translate.

    :param domain: name or alternate path of yaml file.
    :param key: name of main key in yaml file.
    :param language_code: language that you want to translate to.
    :return: str
    """

    # convert file name to file system format.
    file_name = f"{domain.replace("_", "/")}.yml"
    try:
        with importlib.resources.files("mizuhara.translation").joinpath(file_name).open("r", encoding="utf-8") as file:
            content = safe_load(file) or {}
            if content.get(key.lower(), None) is None:
                raise ModuleNotFoundError

    except (FileNotFoundError, ModuleNotFoundError):
        # Check if there's a user-defined translation file
        if exists(f"translation/{file_name}"):
            with open(f"translation/{file_name}", mode="r", encoding="utf-8") as file:
                content = safe_load(file) or {}

        # If no translation file exists, return the original key
        else:
            return key

    return content.get(key.lower(), {}).get(language_code, key)
